@@ -17,12 +17,11 @@ import numpy as np
 import scipy
 import scipy.io
 import torch
-import torch.nn.functional as F
 
 from pyiqa.utils.color_util import to_y_channel
 from pyiqa.utils.download_util import load_file_from_url
-from pyiqa.matlab_utils import imresize, fspecial_gauss, conv2d, imfilter, fitweibull
-from .func_util import estimate_aggd_param, torch_cov, normalize_img_with_guass, nanmean
+from pyiqa.matlab_utils import imresize, fspecial, conv2d, imfilter, fitweibull, nancov, nanmean, blockproc
+from .func_util import estimate_aggd_param, normalize_img_with_guass, diff_round
 from pyiqa.archs.fsim_arch import _construct_filters
 from pyiqa.utils.registry import ARCH_REGISTRY
 
@@ -121,15 +120,7 @@ def niqe(img: torch.Tensor,
     for scale in (1, 2):  # perform on two scales (1, 2)
         img_normalized = normalize_img_with_guass(img, padding='replicate')
 
-        feat = []
-        for idx_w in range(num_block_w):
-            for idx_h in range(num_block_h):
-                # process ecah block
-                block = img_normalized[..., idx_h * block_size_h // scale:(idx_h + 1) * block_size_h // scale,
-                                       idx_w * block_size_w // scale:(idx_w + 1) * block_size_w // scale]
-                feat.append(compute_feature(block))
-
-        distparam.append(torch.stack(feat).transpose(0, 1))
+        distparam.append(blockproc(img_normalized, [block_size_h // scale, block_size_w // scale], fun=compute_feature))
 
         if scale == 1:
             img = imresize(img / 255., scale=0.5, antialiasing=True)
@@ -139,9 +130,7 @@ def niqe(img: torch.Tensor,
 
     # fit a MVG (multivariate Gaussian) model to distorted patch features
     mu_distparam = nanmean(distparam, dim=1)
-
-    distparam_no_nan = torch.nan_to_num(distparam)
-    cov_distparam = torch_cov(distparam_no_nan.transpose(1, 2))
+    cov_distparam = nancov(distparam)
 
     # compute niqe quality, Eq. 10 in the paper
     invcov_param = torch.linalg.pinv((cov_pris_param + cov_distparam) / 2)
@@ -181,6 +170,9 @@ def calculate_niqe(img: torch.Tensor,
     if test_y_channel and img.shape[1] == 3:
         img = to_y_channel(img, 255, color_space)
 
+    img = diff_round(img)
+    img = img.to(torch.float64)
+
     if crop_border != 0:
         img = img[..., crop_border:-crop_border, crop_border:-crop_border]
 
@@ -336,19 +328,10 @@ def ilniqe(img: torch.Tensor,
         GM = torch.cat(GM, dim=1)
         compositeMat = torch.cat((compositeMat, logResponse, partialDer, GM), dim=1)
 
-        feat = []
-        for idx_w in range(num_block_w):
-            for idx_h in range(num_block_h):
-                block_pos = [
-                    idx_h * block_size_h // scale, (idx_h + 1) * block_size_h // scale, idx_w * block_size_w // scale,
-                    (idx_w + 1) * block_size_w // scale
-                ]
-                block = compositeMat[..., block_pos[0]:block_pos[1], block_pos[2]:block_pos[3]]
-                feat.append(compute_feature(block, ilniqe=True))
+        distparam.append(blockproc(compositeMat, [block_size_h // scale,
+                         block_size_w // scale], fun=compute_feature, ilniqe=True))
 
-        distparam.append(torch.stack(feat, dim=1))
-
-        gauForDS = fspecial_gauss(math.ceil(6 * sigmaForDownsample), sigmaForDownsample).to(img)
+        gauForDS = fspecial(math.ceil(6 * sigmaForDownsample), sigmaForDownsample).to(img)
         filterResult = imfilter(O_img, gauForDS.repeat(3, 1, 1, 1), padding='replicate', groups=3)
         O_img = filterResult[..., ::2, ::2]
         filterResult = imfilter(img, gauForDS.repeat(3, 1, 1, 1), padding='replicate', groups=3)
@@ -364,9 +347,7 @@ def ilniqe(img: torch.Tensor,
     b, blk_num, feat_num = final_features.shape
 
     # remove block features with nan and compute nonan cov
-    nan_mask = torch.isnan(final_features).any(dim=2, keepdim=True)
-    final_features_nonan = final_features.masked_select(~nan_mask).reshape(b, -1, feat_num)
-    cov_distparam = torch_cov(final_features_nonan, rowvar=False)
+    cov_distparam = nancov(final_features)
 
     # replace nan in final features with mu
     mu_final_features = nanmean(final_features, dim=1, keepdim=True)
@@ -397,7 +378,7 @@ def calculate_ilniqe(img: torch.Tensor,
 
     params = scipy.io.loadmat(pretrained_model_path)
     img = img * 255.
-    img = img.round()
+    img = diff_round(img)
     # float64 precision is critical to be consistent with matlab codes
     img = img.to(torch.float64)
 
