@@ -9,7 +9,7 @@ import torchvision.transforms as tf
 from torchvision.transforms.functional import normalize
 
 from pyiqa.data.data_util import read_meta_info_file
-from pyiqa.data.transforms import transform_mapping, augment, paired_random_crop
+from pyiqa.data.transforms import transform_mapping, augment
 from pyiqa.utils import FileClient, imfrombytes, img2tensor
 from pyiqa.utils.registry import DATASET_REGISTRY
 
